@@ -182,7 +182,7 @@ backtrace(MaxDepth, Fr, PC, GoalDepth, Guard,
 
 more_stack(Parent) :-
     prolog_frame_attribute(Parent, predicate_indicator, PI),
-    \+ (   PI = '$toplevel':G,
+    \+ (   PI = ('$toplevel':G),
            G \== (toplevel_call/1)
        ),
     !.
@@ -353,7 +353,7 @@ message_frames(frame(Level, _Where, '$toplevel':toplevel_call(_)), _) -->
     [ '<user>'-[] ].
 message_frames(frame(Level, Where, Goal), Options) -->
     level(Level),
-    [ '~q'-[Goal] ],
+    [ '~p'-[Goal] ],
     where_goal(Where, Options).
 
 where_no_goal(foreign(PI), _) -->
@@ -600,6 +600,7 @@ join_stacks(_, Stack, Stack).
 %   from C will handle the exception.
 
 stack_guard(none).
+stack_guard(system:catch_with_backtrace/3).
 
 
                  /*******************************
