@@ -74,8 +74,7 @@ typedef struct
     code multifile[3];			/* S_MULTIFILE */
     code staticp[3];			/* S_STATIC */
     code wrapper[3];			/* S_WRAP */
-    code trie_gen2[3];			/* S_TRIE_GEN2 */
-    code trie_gen3[3];			/* S_TRIE_GEN3 */
+    code trie_gen[3];			/* S_TRIE_GEN */
   } supervisors;
 } PL_code_data_t;
 
@@ -177,9 +176,19 @@ struct PL_global_data
   { Table	modules;		/* atom --> module */
   } tables;
 
-  struct
+#if O_PLMT
+  struct				/* Shared table data */
   { struct trie *variant_table;		/* Variant --> table */
+    trie_allocation_pool node_pool;	/* Node allocation pool for tries */
+    simpleMutex  mutex;			/* Sync completion */
+#ifdef __WINDOWS__
+    win32_cond_t cvar;
+#else
+    pthread_cond_t cvar;
+#endif
+    struct trie_array *waiting;		/* thread --> trie we are waiting for */
   } tabling;
+#endif
 
   struct
   { Table	record_lists;		/* Available record lists */
@@ -316,6 +325,8 @@ struct PL_global_data
 #endif
     Procedure   comment_hook3;		/* prolog:comment_hook/3 */
     Procedure	tune_gc3;		/* prolog:tune_gc */
+    Procedure	trie_gen_compiled2;
+    Procedure	trie_gen_compiled3;
 
     int		static_dirty;		/* #static dirty procedures */
 #ifdef O_CLAUSEGC
