@@ -947,6 +947,8 @@ PRED_IMPL("win_folder", 2, win_folder, PL_FA_NONDETERMINISTIC)
 	  { PL_close_foreign_frame(fid);
 	    ForeignRedoInt(n);
 	  }
+	  if ( PL_exception(0) )
+	    PL_clear_exception();
 	  PL_rewind_foreign_frame(fid);
 	}
 	PL_close_foreign_frame(fid);
@@ -1118,6 +1120,23 @@ getDefaultsFromRegistry(void)
   }
 }
 
+
+const char *
+PL_w32_running_under_wine(void)
+{ static const char * (CDECL *pwine_get_version)(void);
+  HMODULE hntdll = GetModuleHandle("ntdll.dll");
+
+  if ( !hntdll )
+  { return NULL;
+  }
+
+  if ( (pwine_get_version = (void *)GetProcAddress(hntdll, "wine_get_version")) )
+    return pwine_get_version();
+
+  return NULL;
+}
+
+
 		 /*******************************
 		 *      PUBLISH PREDICATES	*
 		 *******************************/
