@@ -99,7 +99,9 @@ setupProlog(void)
   GD->combined_stack.overflow_id = STACK_OVERFLOW;
 
   initPrologLocalData(PASS_LD1);
-  LD->tabling.node_pool.limit = GD->options.tableSpace;
+#ifdef O_PLMT
+  LD->tabling.node_pool.limit = GD->options.sharedTableSpace;
+#endif
 
   DEBUG(1, Sdprintf("Atoms ...\n"));
   initAtoms();
@@ -118,6 +120,8 @@ setupProlog(void)
   initRecords();
   DEBUG(1, Sdprintf("Tries ...\n"));
   initTries();
+  DEBUG(1, Sdprintf("Tabling ...\n"));
+  initTabling();
   DEBUG(1, Sdprintf("Flags ...\n"));
   initFlags();
   DEBUG(1, Sdprintf("Foreign Predicates ...\n"));
