@@ -103,7 +103,6 @@ typedef struct worklist
   cluster      *riac;			/* rightmost inner answer cluster */
   cluster      *free_clusters;		/* clusters to reuse */
   int		magic;			/* WORKLIST_MAGIC */
-  unsigned	completed : 1;		/* Really completed */
   unsigned	ground : 1;		/* Ground call (early completion) */
   unsigned	executing : 1;		/* $tbl_wkl_work/3 in progress */
   unsigned	in_global_wl : 1;	/* already in global worklist */
@@ -186,6 +185,12 @@ typedef struct idg_node
 } idg_node;
 
 
+typedef struct trie_array
+{ trie **blocks[MAX_BLOCKS];
+  trie *preallocated[7];
+} trie_array;
+
+
 		 /*******************************
 		 *	     PROTOTYPES		*
 		 *******************************/
@@ -195,5 +200,6 @@ COMMON(term_t)	init_delay_list(void);
 COMMON(void)	tbl_push_delay(atom_t atrie, Word wrapper,
 			       trie_node *answer ARG_LD);
 COMMON(int)	answer_is_conditional(trie_node *answer);
+COMMON(void)	initTabling(void);
 
 #endif /*_PL_TABLING_H*/
