@@ -96,7 +96,6 @@ typedef struct ukey_state
   Word		var_buf[NVARS_FAST];	/* quick var buffer */
 } ukey_state;
 
-static void	trie_destroy(trie *trie);
 static int	unify_key(ukey_state *state, word key ARG_LD);
 static void	init_ukey_state(ukey_state *state, trie *trie, Word p);
 static void	destroy_ukey_state(ukey_state *state);
@@ -207,9 +206,10 @@ trie_create(void)
 }
 
 
-static void
+void
 trie_destroy(trie *trie)
 { DEBUG(MSG_TRIE_GC, Sdprintf("Destroying trie %p\n", trie));
+  trie->magic = TRIE_CMAGIC;
   trie_empty(trie);
   PL_free(trie);
 }
@@ -217,14 +217,18 @@ trie_destroy(trie *trie)
 
 static void
 trie_discard_clause(trie *trie)
-{ ClauseRef cref;
+{ atom_t dbref;
 
-  if ( (cref=trie->clause) )
-  { Clause cl = cref->value.clause;
+  if ( (dbref=trie->clause) )
+  { if ( COMPARE_AND_SWAP(&trie->clause, dbref, 0) )
+    { ClauseRef cref = clause_clref(dbref);
 
-    if ( COMPARE_AND_SWAP(&trie->clause, cref, NULL) )
-    { set_trie_clause_general_undefined(cl);	/* TBD: only if undefined */
-      retractClauseDefinition(cl->predicate, cl);
+      if ( cref )
+      { Clause cl = cref->value.clause;
+	set_trie_clause_general_undefined(cl);	/* TBD: only if undefined */
+	retractClauseDefinition(cl->predicate, cl);
+      }
+      PL_unregister_atom(dbref);
     }
   }
 }
@@ -232,9 +236,7 @@ trie_discard_clause(trie *trie)
 
 void
 trie_empty(trie *trie)
-{ trie->magic = TRIE_CMAGIC;
-
-  trie_discard_clause(trie);
+{ trie_discard_clause(trie);
 
   if ( !trie->references )
   { indirect_table *it = trie->indirects;
@@ -689,6 +691,8 @@ unify_trie_term(trie_node *node, term_t term ARG_LD)
 	Undo(m);
 	if ( makeMoreStackSpace(rc, ALLOW_GC) )
 	  goto retry;
+	else
+	  return FALSE;
       }
     }
     destroy_ukey_state(&ustate);
@@ -816,10 +820,9 @@ trie_symbol(trie *trie)
 trie *
 symbol_trie(atom_t symbol)
 { void *data;
-  size_t len;
   PL_blob_t *type;
 
-  if ( (data = PL_blob_data(symbol, &len, &type)) && type == &trie_blob )
+  if ( (data = PL_blob_data(symbol, NULL, &type)) && type == &trie_blob )
   { tref *ref = data;
 
     if ( ref->trie->magic == TRIE_MAGIC )
@@ -926,7 +929,8 @@ PRED_IMPL("trie_destroy", 1, trie_destroy, 0)
 { trie *trie;
 
   if ( get_trie(A1, &trie) )
-  { trie_empty(trie);
+  { trie->magic = TRIE_CMAGIC;
+    trie_empty(trie);
 
     return TRUE;
   }
@@ -2384,61 +2388,59 @@ create_trie_clause(Definition def, Clause *cp, trie_compile_state *state)
 }
 
 
-ClauseRef
+atom_t
 compile_trie(Definition def, trie *trie ARG_LD)
-{ ClauseRef rc;
+{ atom_t dbref;
 
 retry:
-  if ( !(rc = trie->clause) )
-  { trie_compile_state state;
-    Clause cl;
+  if ( !(dbref = trie->clause) )
+  { if ( trie->value_count == 0 )
+    { dbref = ATOM_fail;
+      if ( !COMPARE_AND_SWAP(&trie->clause, 0, dbref) )
+	goto retry;
+    } else
+    { trie_compile_state state;
+      Clause cl;
+      ClauseRef cref;
 
-    init_trie_compile_state(&state, trie);
-    add_vmi(&state, def->functor->arity == 2 ? T_TRIE_GEN2 : T_TRIE_GEN3);
-    if ( compile_trie_node(&trie->root, &state PASS_LD) &&
-	 create_trie_clause(def, &cl, &state) )
-    { rc = assertDefinition(def, cl, CL_END PASS_LD);
-      if ( rc )
-      { if ( !COMPARE_AND_SWAP(&trie->clause, NULL, rc) )
-	{ retractClauseDefinition(def, cl);
-	  goto retry;
+      init_trie_compile_state(&state, trie);
+      add_vmi(&state, def->functor->arity == 2 ? T_TRIE_GEN2 : T_TRIE_GEN3);
+      if ( compile_trie_node(&trie->root, &state PASS_LD) &&
+	   create_trie_clause(def, &cl, &state) )
+      { cref = assertDefinition(def, cl, CL_END PASS_LD);
+	if ( cref )
+	{ dbref = lookup_clref(cref->value.clause);
+	  if ( !COMPARE_AND_SWAP(&trie->clause, 0, dbref) )
+	  { PL_unregister_atom(dbref);
+	    retractClauseDefinition(def, cref->value.clause);
+	    goto retry;
+	  }
 	}
       }
+      assert(state.else_loc == 0);
+      clean_trie_compile_state(&state);
     }
-    assert(state.else_loc == 0);
-    clean_trie_compile_state(&state);
   }
 
-  return rc;
+  return dbref;
 }
 
 
-trie *
-get_trie_ptr(Word p ARG_LD)
-{ void *data = NULL;
-  term_t t;
+static
+PRED_IMPL("$trie_compile", 2, trie_compile, 0)
+{ PRED_LD
+  trie *trie;
 
-  deRef(p);
-  if ( isAtom(*p) )
-  { size_t len;
-    PL_blob_t *type;
+  if ( get_trie(A1, &trie) )
+  { Procedure proc = (true(trie, TRIE_ISMAP)
+			     ? GD->procedures.trie_gen_compiled3
+			     : GD->procedures.trie_gen_compiled2);
+    atom_t clref = compile_trie(proc->definition, trie PASS_LD);
 
-    if ( (data = PL_blob_data(*p, &len, &type)) && type == &trie_blob )
-    { tref *ref = data;
-
-      if ( ref->trie->magic == TRIE_MAGIC )
-	return ref->trie;
-    }
+    return _PL_unify_atomic(A2, clref);
   }
 
-  t = pushWordAsTermRef(p);
-  if ( data )
-    PL_existence_error("trie", t);
-  else
-    PL_type_error("trie", t);
-  popTermRef();
-
-  return NULL;
+  return FALSE;
 }
 
 
@@ -2479,6 +2481,7 @@ BeginPredDefs(trie)
   PRED_DEF("trie_gen",            2, trie_gen,      PL_FA_NONDETERMINISTIC)
   PRED_DEF("$trie_gen_node",      3, trie_gen_node, PL_FA_NONDETERMINISTIC)
   PRED_DEF("$trie_property",      2, trie_property,      0)
+  PRED_DEF("$trie_compile",       2, trie_compile,       0)
 EndPredDefs
 
 void
@@ -2491,10 +2494,12 @@ initTries(void)
   proc = PL_predicate("trie_gen_compiled", 2, "system");
   def = proc->definition;
   set(def, P_LOCKED_SUPERVISOR|P_VOLATILE);
-  def->codes = SUPERVISOR(trie_gen2);
+  def->codes = SUPERVISOR(trie_gen);
+  GD->procedures.trie_gen_compiled2 = proc;
 
   proc = PL_predicate("trie_gen_compiled", 3, "system");
   def = proc->definition;
   set(def, P_LOCKED_SUPERVISOR|P_VOLATILE);
-  def->codes = SUPERVISOR(trie_gen3);
+  def->codes = SUPERVISOR(trie_gen);
+  GD->procedures.trie_gen_compiled3 = proc;
 }
