@@ -94,6 +94,8 @@ typedef struct trie_allocation_pool
 
 #define TRIE_ISSET	0x0001		/* Trie nodes have no value */
 #define TRIE_ISMAP	0x0002		/* Trie nodes have a value */
+#define TRIE_ISSHARED	0x0004		/* This is a shared answer trie */
+#define TRIE_COMPLETE	0x0008		/* Answer trie is complete */
 
 typedef struct trie
 { atom_t		symbol;		/* The associated symbol */
@@ -102,11 +104,14 @@ typedef struct trie
   unsigned int		node_count;	/* # nodes */
   unsigned int		value_count;	/* # nodes with a value */
   unsigned int		flags;		/* misc flags */
+#ifdef O_PLMT
+  int			tid;		/* thread id doing completion or re-evaluation */
+#endif
   trie_node	        root;		/* the root node */
   indirect_table       *indirects;	/* indirect values */
   void		      (*release_node)(struct trie *, trie_node *);
   trie_allocation_pool *alloc_pool;	/* Node allocation pool */
-  ClauseRef		clause;		/* Compiled representation */
+  atom_t		clause;		/* Compiled representation */
 #ifdef O_TRIE_STATS
   struct
   { uint64_t		lookups;	/* trie_lookup */
@@ -132,6 +137,7 @@ typedef struct trie
 
 COMMON(void)	initTries(void);
 COMMON(trie *)	trie_create(void);
+COMMON(void)	trie_destroy(trie *trie);
 COMMON(void)	trie_empty(trie *trie);
 COMMON(void)	trie_clean(trie *trie);
 COMMON(void)	trie_delete(trie *trie, trie_node *node, int prune);
@@ -140,7 +146,6 @@ COMMON(trie *)	get_trie_from_node(trie_node *node);
 COMMON(int)	is_ground_trie_node(trie_node *node);
 COMMON(int)	get_trie(term_t t, trie **tp);
 COMMON(int)	get_trie_noex(term_t t, trie **tp);
-COMMON(trie *)  get_trie_ptr(Word p ARG_LD);
 COMMON(int)	unify_trie_term(trie_node *node, term_t term ARG_LD);
 COMMON(int)	trie_lookup(trie *trie, trie_node **nodep, Word k,
 			    int add, TmpBuffer vars ARG_LD);
@@ -156,6 +161,6 @@ COMMON(foreign_t) trie_gen(term_t Trie, term_t Key, term_t Value,
 			   void *ctx, control_t PL__ctx);
 COMMON(void *)	map_trie_node(trie_node *n,
 			      void* (*map)(trie_node *n, void *ctx), void *ctx);
-COMMON(ClauseRef) compile_trie(Definition def, trie *trie ARG_LD);
+COMMON(atom_t)	compile_trie(Definition def, trie *trie ARG_LD);
 
 #endif /*_PL_TRIE_H*/
